﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace NetworkBuilder
{
    public class Calculator
    {
        public static IList<long> Vectorise(IList<long> m, IList<long> l)
        {
            IList<long> o = new List<long>();
            foreach (long p in m)
            {
                if (l.Contains(p))
                    o.Add(1);
                else
                    o.Add(0);
            }

            return o;
        }
        public static IList<long> Vectorise(IList<long> m, IDictionary<long, double> l)
        {
            IList<long> o = new List<long>();
            foreach (long p in m)
            {
                if (l.ContainsKey(p))
                {
                    if (l[p] > 0)
                        o.Add(1);
                    else
                        o.Add(0);
                }
                else
                    o.Add(0);
            }

            return o;
        }
        public static double PearsonCorrelation(IList<long> xs, IList<long> ys)
        {
            // sums of x, y, x squared etc.
            double sx = 0.0;
            double sy = 0.0;
            double sxx = 0.0;
            double syy = 0.0;
            double sxy = 0.0;

            int n = 0;

            using (var enX = xs.GetEnumerator())
            {
                using (var enY = ys.GetEnumerator())
                {
                    while (enX.MoveNext() && enY.MoveNext())
                    {
                        double x = enX.Current;
                        double y = enY.Current;

                        n += 1;
                        sx += x;
                        sy += y;
                        sxx += x * x;
                        syy += y * y;
                        sxy += x * y;
                    }
                }
            }

            // covariation
            double cov = sxy / n - sx * sy / n / n;
            // standard error of x
            double sigmaX = Math.Sqrt(sxx / n - sx * sx / n / n);
            // standard error of y
            double sigmaY = Math.Sqrt(syy / n - sy * sy / n / n);

            // correlation is just a normalized covariation
            return cov / sigmaX / sigmaY;
        }
        public static double PearsonCorrelation(IList<int> xs, IList<int> ys)
        {
            // sums of x, y, x squared etc.
            double sx = 0.0;
            double sy = 0.0;
            double sxx = 0.0;
            double syy = 0.0;
            double sxy = 0.0;

            int n = 0;

            using (var enX = xs.GetEnumerator())
            {
                using (var enY = ys.GetEnumerator())
                {
                    while (enX.MoveNext() && enY.MoveNext())
                    {
                        double x = enX.Current;
                        double y = enY.Current;

                        n += 1;
                        sx += x;
                        sy += y;
                        sxx += x * x;
                        syy += y * y;
                        sxy += x * y;
                    }
                }
            }

            // covariation
            double cov = sxy / n - sx * sy / n / n;
            // standard error of x
            double sigmaX = Math.Sqrt(sxx / n - sx * sx / n / n);
            // standard error of y
            double sigmaY = Math.Sqrt(syy / n - sy * sy / n / n);

            // correlation is just a normalized covariation
            return cov / sigmaX / sigmaY;
        }

        public static double Log(double x)
        {
            if (x == 0) return 0;
            double v = Math.Log(x, 10);
            if (Double.IsNaN(v)) return 0;
            else return v;
        }


        public static double Sum(IDictionary<long, double> P)
        {
            double max = 0.0;
            foreach (long p in P.Keys)
                max += P[p];
            return max;
        }


        public static double Entropy(IDictionary<long, double> P)
        {
            double max = Sum(P);
            if (max == 0) return 0;

            double value = 0.0;
            foreach (long j in P.Keys)
            {
                double pij = P[j] / max;
                double entropy = -1 * pij * Calculator.Log(pij);
                if (!Double.IsNaN(entropy))
                    value += entropy;
            }
            return value;

        }

    }
}
