﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;

namespace NetworkBuilder
{
    public class Edge
    {
        //Basic
        public int SourceIndex { get; set; }
        public int TargetIndex { get; set; }
        public IList<long> Pathogens { get; set; }

        //constructor
        public Edge(int Source, int Target, IList<long> P)
        {
            this.SourceIndex = Source;
            this.TargetIndex = Target;
            this.Pathogens = new List<long>();
            foreach (long p in P)
            {
                if (!Pathogens.Contains(p))
                    Pathogens.Add(p);
            }

        }

        //helpers     
        public string PathogensList()
        {
            if (this.Pathogens == null) return "";
            if (this.Pathogens.Count == 0) return "";
            StringBuilder sb = new StringBuilder();
            sb.Append(this.Pathogens[0]);
            for (int i = 1; i < this.Pathogens.Count; i++)
            {
                sb.Append(";");
                sb.Append(this.Pathogens[i]);
            }
            return sb.ToString();
        }

        //output
        public void Write(TextWriter tw, IList<Node> Nodes)
        {
            if (this.Pathogens.Count > 0)
            {
                int i = this.SourceIndex;
                int j = this.TargetIndex;
                if (Nodes[this.SourceIndex].EID2ID < Nodes[this.TargetIndex].EID2ID)
                {
                    j = this.SourceIndex;
                    i = this.TargetIndex;
                }
                tw.WriteLine(String.Format("{0},{1},{2},{3},{4}",
                                                          Nodes[i].Name, Nodes[j].Name,
                                                          Nodes[i].EID2ID, Nodes[j].EID2ID,
                                                          this.Pathogens.Count));
                tw.Flush();
            }
        }
        public void WritePathogens(TextWriter tw, IList<Node> Nodes)
        {
            int i = this.SourceIndex;
            int j = this.TargetIndex;

            if (Nodes[this.SourceIndex].EID2ID < Nodes[this.TargetIndex].EID2ID)
            {
                j = this.SourceIndex;
                i = this.TargetIndex;
            }

            if (this.Pathogens.Count > 0)
            {
                tw.WriteLine(String.Format("{0}\t{1}\t{2}",
                                     Nodes[i].EID2ID, Nodes[j].EID2ID, this.PathogensList()));
                tw.Flush();
            }
        }
    }
}
