﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace NetworkBuilder
{
    public class Graph
    {
        public IList<Node> Nodes { get; set; }
        public IList<Edge> Edges { get; set; }
        public IDictionary<long, IList<long>> Neighbours { get; set; }
        public IDictionary<string, IList<long>> EdgePathogens { get; set; }
        public IDictionary<long, double> Degrees { get; set; }
        public IDictionary<long, double> WeightedDegrees { get; set; }
        public double Entropy { get; set; }
        public double WeightedEntropy { get; set; }
        //public int P { get; set; } //all pathogens = all shared messages
        public int S { get; set; } //all shared pathogens = all shared messages
        public int N { get; set; }

        public Graph()
        {
            this.Nodes = new List<Node>();
            this.Edges = new List<Edge>();
        }

        public Graph(IList<Node> nodes, IList<Edge> edges)
        {
            this.Nodes = new List<Node>();
            foreach (Node n in nodes)
                this.Nodes.Add(n);

            this.Edges = new List<Edge>();
            foreach (Edge e in edges)
                this.Edges.Add(e);

            this.Neighbours = NetworkProperties.Neighbours(edges, nodes);
            this.EdgePathogens = NetworkProperties.EdgePathogens(edges, nodes);
            this.WeightedDegrees = NetworkProperties.WeightedDegrees(nodes, Neighbours, EdgePathogens);
            this.Degrees = NetworkProperties.Degrees(nodes, Neighbours, EdgePathogens);

            IList<long> AllSharedPathogens = Helper.SharedPathogens(edges);            
            this.S = AllSharedPathogens.Count;
            this.N = Nodes.Count;          
        }

        public IList<long> SharedNeighbours(long i, long j)
        {
            IEnumerable<long> l = this.Neighbours[i].Intersect<long>(this.Neighbours[j]).ToList<long>();
            if (l != null)
                return l.ToList<long>();
            return new List<long>();
        }
    }
}
