﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;


namespace NetworkBuilder
{
    public class IOHelper
    {
        
        public static string GenerateNetworkFolder(string pathogenType, string pathogenSubType)
        {
            //\\ch-fs01\RDM02\BigDataapproaches\networks of vert\data\main networks\mammals (no rodents)
            if (pathogenType == "")
                pathogenType = "all";

            string mainDirectory = String.Format(@"{0}\{1}", Globals.OutputFolder, pathogenType);
            if (!Directory.Exists(mainDirectory))
                Directory.CreateDirectory(mainDirectory);

            string directory = String.Format(@"{0}\{1}", mainDirectory, pathogenSubType);
            if (!Directory.Exists(directory))
            {
                Directory.CreateDirectory(directory);
            }

            return directory;
        }

        public static string GenerateEnumeratorFolder(string pathogenType, string pathogenSubType, int sample)
        {
            string folder = @"../../output/permutations/networks";
           
            if (pathogenType == "")
                pathogenType = "all";

            string mainDirectory = String.Format(@"{0}\{1}", folder, pathogenType);
            if (!Directory.Exists(mainDirectory))
                Directory.CreateDirectory(mainDirectory);

            string directory = String.Format(@"{0}\{1}", mainDirectory, pathogenSubType);
            if (!Directory.Exists(directory))
            {
                Directory.CreateDirectory(directory);
            }

            directory = String.Format(@"{0}\{1}", directory, sample);
            if (!Directory.Exists(directory))
            {
                Directory.CreateDirectory(directory);
            }

            return directory;
        }

        public static string GenerateBasicsFolder(string pathogenSubType)
        {
            string folder = @"../../output/permutations/basics";
            if (pathogenSubType == "")
                pathogenSubType = "all";

            string directory = String.Format(@"{0}\{1}", folder, pathogenSubType);
            if (!Directory.Exists(directory))
            {
                Directory.CreateDirectory(directory);
            }

            
            return directory;
        }


        public static IDictionary<string, int> Header_CSV(string header)
        {
            IDictionary<string, int> dic = new Dictionary<string, int>();
            string[] items = header.ToLower().Split(',');
            for (int i = 0; i < items.Length; i++)
            {
                if (items[i].Trim().Trim('\"').Trim() != "")
                {
                    dic.Add(items[i].Trim().Trim('\"').Trim(), i);
                }
            }
            return dic;
        }

        public static int Index(IDictionary<string, int> header, string key)
        {
            if (!header.ContainsKey(key.ToLower())) return -1;
            return header[key.ToLower()];
        }

        public static string Value_string(IDictionary<string, int> header, string key, string[] items)
        {
            int index = Index(header, key);
            if (index < 0) return "";
            else return items[index].Trim().Trim('\"').Trim();
        }

        public static int Value_int(IDictionary<string, int> header, string key, string[] items)
        {
            int index = Index(header, key);
            if (index < 0) return -1;
            else return Convert.ToInt32(items[index].Trim().Trim('\"').Trim());
        }

        public static long Value_long(IDictionary<string, int> header, string key, string[] items)
        {
            int index = Index(header, key);
            if (index < 0) return -1;
            else return Convert.ToInt64(items[index].Trim().Trim('\"').Trim());
        }

        public static double Value_double(IDictionary<string, int> header, string key, string[] items)
        {
            int index = Index(header, key);
            if (index < 0) return -1.0;
            else return Convert.ToDouble(items[index].Trim().Trim('\"').Trim());
        }
    }
}
