﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;

namespace NetworkBuilder
{
    public class NeighbourHelper
    {
        static public string to_string(IList<long> nb)
        {
            if (nb == null) return "";
            if (nb.Count == 0) return "";
            StringBuilder sb = new StringBuilder();
            sb.Append(nb[0]);
            for (int i = 1; i < nb.Count; i++)
            {
                sb.Append(";");
                sb.Append(nb[i]);
            }
            return sb.ToString();
        }

      

        static public void Neighbours(string P, string PT)
        {
            IList<long> Pathogens = PathogensExtractor.ExtractSubType(P, PT);
            //Nodes first run
            Console.WriteLine("generating nodes - first run");
            var watch = System.Diagnostics.Stopwatch.StartNew();
            IList<Node> nodes = NetworkBuilder.GenerateNodes(P, PT);
            watch.Stop();
            string elapsed = watch.Elapsed.ToString();
            Console.WriteLine(elapsed);

            //Edges 
            Console.WriteLine("generating edges - first run");
            watch = System.Diagnostics.Stopwatch.StartNew();
            IList<Edge> edges = NetworkBuilder.GenerateEdges(nodes);
            elapsed = watch.Elapsed.ToString();
            Console.WriteLine(elapsed);


            Console.WriteLine("generating graph");
            watch = System.Diagnostics.Stopwatch.StartNew();
            Graph g = new Graph(nodes, edges);
            Console.WriteLine(elapsed);

          

            string neighbours_file = String.Format(@"../../output/neighbours/{0}_{1}_nb.csv", P, PT);
            TextWriter tw = new StreamWriter(neighbours_file, false);
            tw.WriteLine(string.Format("{0},{1}", "node", "neighbours"));
            tw.Flush();

            foreach (long n in g.Neighbours.Keys)
            {
                string s = to_string(g.Neighbours[n]);
                tw.WriteLine(string.Format("{0},{1}", n, s));
                tw.Flush();
            }
            tw.Close();
        }

        public static IDictionary<string, double> GetDistances(int indx)
        {
            IDictionary<string, double> dic = new Dictionary<string, double>();
            string[] Data = File.ReadAllLines(@"../../data/all_distances.csv");
            //node1	node2	distance_v	distance_p

            for (int i = 1; i < Data.Length; i++)
            {
                try
                {
                    string[] items = Data[i].Split(',');
                    long id1 = Convert.ToInt64(items[0].Trim());
                    long id2 = Convert.ToInt64(items[1].Trim());
                    double distance = Convert.ToDouble(items[indx].Trim());

                    string key = Helper.GetKey(id1, id2);
                    if (!dic.ContainsKey(key))
                        dic.Add(key, distance);
                }
                catch (Exception e)
                {
                    Console.WriteLine(e.Message);
                }
            }
            return dic;
        }

        static private IList<long> to_list(string s)
        {
            IList<long> l = new List<long>();
            if (s != "")
            {
                string[] items = s.Split(';');
                foreach (string i in items)
                {
                    if (i.Trim() != "")
                    {
                        l.Add(Convert.ToInt64(i.Trim()));
                    }
                }
            }
            return l;

        }
        static public double[] Betadiversity(long node, IList<long> neighbours, IDictionary<string, double> dic)
        {
            double[] output = { 0.0, 0.0 };
            if (neighbours == null)
            {
                return output;
            }
            if (neighbours.Count <= 1)
            {
                return output;
            }

            double s = neighbours.Count;
            double w = 0;
            double w2 = 0;

            for (int i = 0; i < s - 1; i++)
            {
                for (int j = i + 1; j < s; j++)
                {
                    if (i != j)
                    {
                        string key = Helper.GetKey(neighbours[i], neighbours[j]);
                        if (dic.ContainsKey(key))
                        {
                            w = w + dic[key];
                            w2 = w2 + Math.Pow(dic[key], 2);
                        }
                    }
                }
            }

            output[0] = 2 * w / (s * (s - 1));
            output[1] = (2 * w2 / (s * (s - 1))) - Math.Pow(output[0], 2);

            return output;
        }

        static public void QuickNeighbours(string P, string PT)
        {
            IDictionary<string, double> vdis = GetDistances(2);
            IDictionary<string, double> pdis = GetDistances(3);

            string output_file = String.Format(@"../../output/distances/{0}_{1}_neighbours.csv", P, PT);

            string neighbours_file = String.Format(@"../../output/neighbours/{0}_{1}_nb.csv", P, PT);          
            if (File.Exists(neighbours_file))
            {
                TextWriter tw = new StreamWriter(output_file, false);
                tw.WriteLine(string.Format("{0},{1},{2},{3},{4}", "node", "pv", "pp", "vpv", "vpp"));
                tw.Flush();

                string[] data = File.ReadAllLines(neighbours_file);


                for (int i = 1; i < data.Length; i++)
                {
                    string[] items = data[i].Split(',');
                    long node = Convert.ToInt64(items[0].Trim());
                    IList<long> neighbours = to_list(items[1].Trim());


                    double[] p_distances = Betadiversity(node, neighbours, pdis);
                    double[] v_distances = Betadiversity(node, neighbours, vdis);

                    tw.WriteLine(string.Format("{0},{1},{2},{3},{4}", node, v_distances[0], p_distances[0], v_distances[1], p_distances[1]));
                    tw.Flush();
                }
                tw.Close();
            }
        }
    }
}
