﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace NetworkBuilder
{
    public class NetworkBuilder
    {

        static public void GenerateNetwork(string P, string PT, IDictionary<string, int> PathogensTaxonomicDistances)
        {
            //vertebrates_nohumans(viruses)_nodes            
            string edges_file = String.Format(@"{0}/edges.csv", IOHelper.GenerateNetworkFolder(P, PT));
            string nodes_file = String.Format(@"{0}/nodes.csv", IOHelper.GenerateNetworkFolder(P, PT));
          
            //Nodes first run
            Console.WriteLine("generating nodes");
            var watch = System.Diagnostics.Stopwatch.StartNew();
            IList<Node> nodes = GenerateNodes(P, PT);
            watch.Stop();
            string elapsed = watch.Elapsed.ToString();
            Console.WriteLine(elapsed);

          

            //Edges 
            Console.WriteLine("generating edges");
            watch = System.Diagnostics.Stopwatch.StartNew();
            IList<Edge> edges = GenerateEdges(nodes);
            elapsed = watch.Elapsed.ToString();
            Console.WriteLine(elapsed);

            /*Console.WriteLine("outputting edges");

            TextWriter tw_edges = new StreamWriter(edges_file, false);
            tw_edges.WriteLine(Globals.EdgesHeader);
            tw_edges.Flush();


            foreach (Edge e in edges)
            {
                e.Write(tw_edges, nodes);
            }
            */

            Console.WriteLine("generating graph");
            watch = System.Diagnostics.Stopwatch.StartNew();
            Graph g = new Graph(nodes, edges);
            Console.WriteLine(elapsed);

           
            Console.WriteLine("calculating diversity &centralities");
            watch = System.Diagnostics.Stopwatch.StartNew();
            IList<long> AllPathogens = Helper.AllPathogens(nodes);
            IList<long> HumanPathogens = TraitsProcessor.GetHumanPathogen(P);

            foreach (Node n in nodes)
            {
                n.SharedWithHumans(HumanPathogens, AllPathogens);
                n.BetaDiversity(PathogensTaxonomicDistances);
                n.Centrality(g);
                n.PrepareSharedPathogens(g);
            }
            elapsed = watch.Elapsed.ToString();
            Console.WriteLine(elapsed);

            Console.WriteLine("outputting nodes");
            TextWriter tw_nodes = new StreamWriter(nodes_file, false);
            tw_nodes.WriteLine(Globals.NodesHeader);
            tw_nodes.Flush();

            foreach (Node n in nodes)
            {
                n.WriteFullNode(tw_nodes);
            }

        }

        static public IList<Node> GenerateNodes(string P, string PT)
        {
            IDictionary<long, string> Order = TraitsProcessor.OrderFamily(3);

            IList<long> Pathogens =  PathogensExtractor.ExtractSubType(P, PT);


            string input = Globals.VertebratesDataSet;

            IList<Node> nodes = new List<Node>();
            string[] Data = File.ReadAllLines(input);

            for (int i = 1; i < Data.Length; i++)
            {
                try
                {
                    nodes = Node.Update(nodes, "mammals", false,Data[i], Pathogens);
                }
                catch (Exception e)
                {
                    Console.WriteLine(e);
                }
            }

            //node attributes
            foreach (Node node in nodes)
            {
                if (Order.ContainsKey(node.EID2ID))
                {
                    node.Order = Order[node.EID2ID];
                }
            }
            return nodes;
        }


        static public IList<Edge> GenerateEdges(IList<Node> nodes)
        {
            IList<Edge> edges = new List<Edge>();
            for (int i = 0; i < nodes.Count - 1; i++)
            {
                for (int j = i + 1; j < nodes.Count; j++)
                {
                    IList<long> shared = nodes[i].SharedInteractions(nodes[j]);
                    if (shared != null && shared.Count > 0)
                    {
                        Edge E = new Edge(i, j, shared);
                        edges.Add(E);
                    }
                }
            }
            return edges;

        }

    }
}
