﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace NetworkBuilder
{
    public class NetworkProperties
    {
        static public IDictionary<long, IList<long>> Neighbours(IList<Edge> Edges, IList<Node> Nodes)
        {
            IDictionary<long, IList<long>> dic = new Dictionary<long, IList<long>>();
            foreach (Edge E in Edges)
            {
                try
                {
                    long id1 = Nodes[E.SourceIndex].EID2ID;
                    long id2 = Nodes[E.TargetIndex].EID2ID;

                    if (!dic.ContainsKey(id1))
                    {
                        dic.Add(id1, new List<long>());
                    }
                    if (!dic.ContainsKey(id2))
                    {
                        dic.Add(id2, new List<long>());
                    }

                    if (!dic[id1].Contains(id2))
                        dic[id1].Add(id2);

                    if (!dic[id2].Contains(id1))
                        dic[id2].Add(id1);
                }
                catch (Exception e)
                {
                    Console.WriteLine(e.Message);
                }
            }
            return dic;
        }
        public static IDictionary<string, IList<long>> EdgePathogens(IList<Edge> Edges, IList<Node> Nodes)
        {
            IDictionary<string, IList<long>> dic = new Dictionary<string, IList<long>>();
            foreach (Edge E in Edges)
            {
                string key = Helper.GetKey(Nodes[E.SourceIndex].EID2ID, Nodes[E.TargetIndex].EID2ID);
                if (!dic.ContainsKey(key))
                {
                    dic.Add(key, new List<long>());
                    foreach (long p in E.Pathogens)
                    {
                        dic[key].Add(p);
                    }
                }
            }
            return dic;
        }

        public static IDictionary<long, double> Degrees(IList<Node> nodes, IDictionary<long, IList<long>> Neighbours, IDictionary<string, IList<long>> EdgePathogens)
        {
            IDictionary<long, double> dic = new Dictionary<long, double>();
            foreach (Node n in nodes)
            {
                if (Neighbours.ContainsKey(n.EID2ID))
                {
                    dic.Add(n.EID2ID, Neighbours[n.EID2ID].Count);
                }
                else
                    dic.Add(n.EID2ID, 0);
            }
            return dic;
        }
        public static IDictionary<long, double> WeightedDegrees(IList<Node> nodes, IDictionary<long, IList<long>> Neighbours, IDictionary<string, IList<long>> EdgePathogens)
        {
            IDictionary<long, double> dic = new Dictionary<long, double>();
            foreach (Node n in nodes)
            {
                if (Neighbours.ContainsKey(n.EID2ID))
                {
                    dic.Add(n.EID2ID, WeightedDegree(n.EID2ID, Neighbours[n.EID2ID], EdgePathogens));
                }
                else
                    dic.Add(n.EID2ID, 0);
            }
            return dic;
        }
        public static double WeightedDegree(long n, IList<long> Neighbours, IDictionary<string, IList<long>> EdgePathogens)
        {
            double wd = 1;
            foreach (long neighbour in Neighbours)
            {
                string k = Helper.GetKey(n, neighbour);
                if (EdgePathogens.ContainsKey(k))
                {
                    wd = wd + EdgePathogens[k].Count;
                }
            }
            return wd;
        }


    }
}
