﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Threading.Tasks;

namespace NetworkBuilder
{
    public class Node
    {
        //node basic attributes
        public string Name { get; set; }
        public long EID2ID { get; set; }
        public long TaxID { get; set; }
        public string Order { get; set; }
        public IList<long> Pathogens { get; set; }
        public IList<long> SharedPathogens { get; set; }
        public int sharecount { get; set; }

        //beta diversity
        public double TDS { get; set; }
        public double VarTDS { get; set; }

        //humans
        public int PathogensSharedWithHumans { get; set; }


        public double IndirectInfluence { get; set; }
        public double IndirectInfluence_deg { get; set; }
        public double IndirectInfluence_freq { get; set; }
        public double IndirectInfluence_mes { get; set; }

        //Constructor
        public Node(string line)
        {
            string[] items = line.ToLower().Split('\t');
            this.Name = items[7].Trim('\"').Trim().Replace("'", "");
            this.EID2ID = Convert.ToInt64(items[8].Trim('\"').Trim());
            this.TaxID = Convert.ToInt64(items[9].Trim('\"').Trim());
            this.Pathogens = new List<long>();
            this.Pathogens.Add(Convert.ToInt64(items[2].Trim('\"').Trim()));

            this.TDS = 0;
            this.VarTDS = 0;

            //Humans
            this.PathogensSharedWithHumans = 0;

            //additional centralities
            this.IndirectInfluence = 0;
            this.IndirectInfluence_deg = 0;
            this.IndirectInfluence_freq = 0;
            this.IndirectInfluence_mes = 0;
        }


        public static IList<Node> Update(IList<Node> L, string HC,
            bool Humans,
            string line, IList<long> Pathogens)
        {
            long[] excluded_rodents = { 3434366, 3434407, 3434405 };
            string[] items = line.ToLower().Split('\t');
            long EID2ID = Convert.ToInt64(items[8].Trim('\"').Trim());
            long pathogen = Convert.ToInt64(items[2].Trim('\"').Trim());
            string pc = items[6].Trim('\"').Trim();
            string hc1 = items[11].Trim('\"').Trim();
            string hc2 = items[12].Trim('\"').Trim();
            if (Pathogens.Contains(pathogen))
            {
                if (HC != "" && HC != hc2) return L;

                if (!Humans && hc1 == "human")
                    return L;
                if (!Humans && EID2ID == 3433574)
                    return L;
                if (excluded_rodents.Contains(EID2ID))
                    return L;
                for (int i = 0; i < L.Count; i++)
                {
                    if (EID2ID == L[i].EID2ID)
                    {
                        if (!L[i].Pathogens.Contains(pathogen))
                        {
                            L[i].Pathogens.Add(pathogen);
                        }
                        return L;
                    }
                }

                Node n = new Node(line);
                L.Add(n);
            }
            return L;
        }


        public static IList<Node> UpdateInteractions(IList<Node> L, long[][] interactions)
        {
            for (int i = 0; i < L.Count; i++)
            {
                for (int j = 0; j < interactions.Length; j++)
                {
                    if (interactions[j][0] == L[i].EID2ID)
                    {
                        if (!L[i].Pathogens.Contains(interactions[j][1]))
                        {
                            L[i].Pathogens.Add(interactions[j][1]);
                        }
                    }
                }
            }
            return L;
        }

        //basic functions
        public void SharedWithHumans(IList<long> humanPathogens, IList<long> allPathogens)
        {
            IList<long> shared = this.Pathogens.Intersect<long>(humanPathogens).ToList<long>();
            if (shared != null)
            {
                this.PathogensSharedWithHumans = shared.Count();
            }
        }

        public IList<long> SharedInteractions(Node N)
        {
            return this.Pathogens.Intersect<long>(N.Pathogens).ToList<long>();
        }


        //helpers     
        public string PathogensList()
        {
            if (this.Pathogens == null) return "";
            if (this.Pathogens.Count == 0) return "";
            StringBuilder sb = new StringBuilder();
            sb.Append(this.Pathogens[0]);
            for (int i = 1; i < this.Pathogens.Count; i++)
            {
                sb.Append(";");
                sb.Append(this.Pathogens[i]);
            }
            return sb.ToString();
        }

        public void GetBetaDiversity(IDictionary<string, int> dic)
        {
            if (this.Pathogens == null)
            {
                this.TDS = 0;
                this.VarTDS = 0;
                return;
            }
            if (this.Pathogens.Count == 1)
            {
                this.TDS = 0;
                this.VarTDS = 0;
                return;
            }

            double s = this.Pathogens.Count;
            double w = 0;
            double w2 = 0;

            for (int i = 0; i < s - 1; i++)
            {
                for (int j = i + 1; j < s; j++)
                {
                    if (i != j)
                    {
                        string key = Helper.GetKey(this.Pathogens[i], this.Pathogens[j]);
                        if (dic.ContainsKey(key))
                        {
                            w = w + dic[key];
                            w2 = w2 + Math.Pow(dic[key], 2);
                        }
                    }
                }
            }

            this.TDS = 2 * w / (s * (s - 1));
            this.VarTDS = (2 * w2 / (s * (s - 1))) - Math.Pow(this.TDS, 2);
        }

        public void BetaDiversity(IDictionary<string, int> dic)
        {
            this.GetBetaDiversity(dic);
        }


        //Output
        public void WriteFullNode(TextWriter tw)
        {
            if (this.Pathogens.Count > 0)
            {
                tw.WriteLine(String.Format("{0},{1},{2},{3},{4},{5},{6},{7},{8},{9}",
                                     this.Name, this.EID2ID, this.TaxID,
                                     this.Order,
                                       this.Pathogens.Count, this.SharedPathogens.Count,
                                     this.TDS, this.VarTDS,
                                     this.IndirectInfluence,
                                     this.PathogensSharedWithHumans
                                     ));
                tw.Flush();
            }
        }
        public void WritePathogens(TextWriter tw)
        {

            if (this.Pathogens.Count > 0)
            {
                tw.WriteLine(String.Format("{0}\t{1}\t{2}",
                                     this.EID2ID, this.TaxID, this.PathogensList()));
                tw.Flush();
            }
        }



        //Centrality Calculations
        public void Centrality(Graph g)
        {
            this.IndirectInfluenceCentrality(g);
        }

        public void IndirectInfluenceCentrality(Graph g)
        {
            if (g.Neighbours.ContainsKey(this.EID2ID))
            {
                double ii_mes = 0;
                double ii_deg = 0;
                double ii_freq = 0;

                IDictionary<long, double> P_Messages = new Dictionary<long, double>();
                IDictionary<long, double> P_Degree = new Dictionary<long, double>();
                IDictionary<long, double> P_Frequencey = new Dictionary<long, double>();

                IList<long> SharedNodes = new List<long>();
                IList<long> SharedMessages = new List<long>();
                double SharedFrequencies = 0.0;

                foreach (long j in g.Neighbours[this.EID2ID])
                {
                    string key_ij = Helper.GetKey(this.EID2ID, j);
                    IList<long> N_ij = g.SharedNeighbours(this.EID2ID, j);

                    if (N_ij.Count > 0)
                    {
                        foreach (long n in N_ij)
                        {
                            if (!SharedNodes.Contains(n))
                                SharedNodes.Add(n);
                        }

                        P_Degree.Add(j, N_ij.Count);
                        if (g.EdgePathogens.ContainsKey(key_ij))
                        {
                            IList<long> S_ij = new List<long>();
                            double shared_ij = 0.0;

                            foreach (long k in N_ij)
                            {
                                string key_jk = Helper.GetKey(j, k);
                                string key_ik = Helper.GetKey(this.EID2ID, k);
                                if (g.EdgePathogens.ContainsKey(key_jk) && g.EdgePathogens.ContainsKey(key_ik))
                                {

                                    IList<long> ik = g.EdgePathogens[key_ik];
                                    IList<long> jk = g.EdgePathogens[key_jk];
                                    IEnumerable<long> ijk = ik.Intersect(jk);
                                    if (ijk != null && ijk.ToList<long>().Count > 0)
                                    {
                                        shared_ij += ((double)ijk.ToList<long>().Count);
                                        SharedFrequencies += ((double)ijk.ToList<long>().Count);
                                        foreach (long p in ijk.ToList<long>())
                                        {
                                            if (!S_ij.Contains(p))
                                                S_ij.Add(p);
                                            if (!SharedMessages.Contains(p))
                                                SharedMessages.Add(p);
                                        }
                                    }
                                }
                            }
                            if (S_ij.Count > 0)
                                P_Messages.Add(j, S_ij.Count);
                            P_Frequencey.Add(j, shared_ij);
                        }
                    }
                }



                ii_deg = Calculator.Entropy(P_Degree) / Calculator.Log(g.N);
                ii_mes = Calculator.Entropy(P_Messages) / Calculator.Log(g.S);
                ii_freq = Calculator.Entropy(P_Frequencey) / Calculator.Log(g.S * g.N);

                this.IndirectInfluence_mes = ii_mes;
                this.IndirectInfluence_deg = ii_deg;
                this.IndirectInfluence_freq = ii_freq;
                this.IndirectInfluence = (this.IndirectInfluence_mes + this.IndirectInfluence_deg + this.IndirectInfluence_freq) / 3;
            }
        }


        public void Qualifiers(Graph g)
        {
            if (g.Neighbours.ContainsKey(this.EID2ID))
            {


                IList<long> SharedNodes = new List<long>();
                IList<long> SharedMessages = new List<long>();
                double SharedFrequencies = 0.0;

                foreach (long j in g.Neighbours[this.EID2ID])
                {
                    string key_ij = Helper.GetKey(this.EID2ID, j);
                    IList<long> N_ij = g.SharedNeighbours(this.EID2ID, j);

                    if (N_ij.Count > 0)
                    {
                        foreach (long n in N_ij)
                        {
                            if (!SharedNodes.Contains(n))
                                SharedNodes.Add(n);
                        }


                        if (g.EdgePathogens.ContainsKey(key_ij))
                        {
                            IList<long> S_ij = new List<long>();
                            double shared_ij = 0.0;

                            foreach (long k in N_ij)
                            {
                                string key_jk = Helper.GetKey(j, k);
                                string key_ik = Helper.GetKey(this.EID2ID, k);
                                if (g.EdgePathogens.ContainsKey(key_jk) && g.EdgePathogens.ContainsKey(key_ik))
                                {

                                    IList<long> ik = g.EdgePathogens[key_ik];
                                    IList<long> jk = g.EdgePathogens[key_jk];
                                    IEnumerable<long> ijk = ik.Intersect(jk);
                                    if (ijk != null && ijk.ToList<long>().Count > 0)
                                    {
                                        shared_ij += ((double)ijk.ToList<long>().Count);
                                        SharedFrequencies += ((double)ijk.ToList<long>().Count);
                                        foreach (long p in ijk.ToList<long>())
                                        {
                                            if (!S_ij.Contains(p))
                                                S_ij.Add(p);
                                            if (!SharedMessages.Contains(p))
                                                SharedMessages.Add(p);
                                        }
                                    }
                                }
                            }

                        }
                    }
                }
                this.IndirectInfluence = (this.IndirectInfluence_mes + this.IndirectInfluence_deg + this.IndirectInfluence_freq) / 3;
            }
        }


        public void PrepareSharedPathogens(Graph g)
        {
            this.SharedPathogens = new List<long>();
            if (g.Neighbours.ContainsKey(this.EID2ID))
            {
                foreach (long j in g.Neighbours[this.EID2ID])
                {
                    string key_ij = Helper.GetKey(this.EID2ID, j);
                    if (g.EdgePathogens.ContainsKey(key_ij))
                    {
                        IList<long> ij = g.EdgePathogens[key_ij];
                        foreach (long p in ij)
                        {
                            if (!this.SharedPathogens.Contains(p))
                                this.SharedPathogens.Add(p);
                        }
                    }
                }
            }
            this.sharecount = this.SharedPathogens.Count;
        }
    }
}
