﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace NetworkBuilder
{
    public class PaperHelper
    {
    

        static public void GenerateNetworkstats(string P, string PT,
            IList<long> Pathogens, TextWriter tw)
        {
            tw.WriteLine(String.Format("{0}\t{1}\t{2}\t{3}\t{4}\t{5}\t{6}\t{7}\t{8}",
                P, PT, "nodes", "shared_human_nodes", "pathogens", "zoonoses", "interactions",
                "interactions_shared", "interactions_zoonoses"));
            tw.Flush();

            //Nodes first run
            Console.WriteLine("generating nodes");
            var watch = System.Diagnostics.Stopwatch.StartNew();
            IList<Node> nodes = NetworkBuilder.GenerateNodes(P, PT);
            watch.Stop();

            IList<long> AllPathogens = Helper.AllPathogens(nodes);
            IList<long> HumanPathogens = TraitsProcessor.GetHumanPathogen(P);
            IList<long> sharing = new List<long>();

            IList<long> allp = new List<long>();
            long alli = 0;
            long alln = 0;
            long allz = 0;
            IList<long> zoonosesP = new List<long>();


            Console.WriteLine("generating edges");
            watch = System.Diagnostics.Stopwatch.StartNew();
            IList<Edge> edges = NetworkBuilder.GenerateEdges(nodes);

            Console.WriteLine("generating graph");
            watch = System.Diagnostics.Stopwatch.StartNew();
            Graph g = new Graph(nodes, edges);


            foreach (Node n in nodes)
            {
                n.SharedWithHumans(HumanPathogens, AllPathogens);
                if (n.PathogensSharedWithHumans > 0)
                    if (!sharing.Contains(n.EID2ID))
                        sharing.Add(n.EID2ID);

                n.PrepareSharedPathogens(g);
                alli += n.Pathogens.Count;
                alln += n.SharedPathogens.Count;

                allz += n.SharedPathogens.Intersect(HumanPathogens).ToList().Count;
                foreach (long p in n.Pathogens)
                {
                    if (!allp.Contains(p))
                        allp.Add(p);
                    if (HumanPathogens.Contains(p))
                        if (!zoonosesP.Contains(p))
                            zoonosesP.Add(p);
                }
            }


            tw.WriteLine(String.Format("{0}\t{1}\t{2}\t{3}\t{4}\t{5}\t{6}\t{7}\t{8}",
                P, PT, nodes.Count, sharing.Count, allp.Count, zoonosesP.Count, alli,
               alln, allz));
            tw.Flush();
        }


        static public string GenerateNetworkstats_string(string P, string PT)
        {
            IList<long> Pathogens = PathogensExtractor.ExtractSubType(P, PT);
            //Nodes first run
            Console.WriteLine("generating nodes");
            var watch = System.Diagnostics.Stopwatch.StartNew();
            IList<Node> nodes = NetworkBuilder.GenerateNodes(P, PT);
            watch.Stop();

            IList<long> AllPathogens = Helper.AllPathogens(nodes);
            IList<long> HumanPathogens = TraitsProcessor.GetHumanPathogen(P);
            IList<long> sharing = new List<long>();

            IList<long> allp = new List<long>();
            long alli = 0;
            long alln = 0;
            long allz = 0;
            IList<long> zoonosesP = new List<long>();


            Console.WriteLine("generating edges");
            watch = System.Diagnostics.Stopwatch.StartNew();
            IList<Edge> edges = NetworkBuilder.GenerateEdges(nodes);

            Console.WriteLine("generating graph");
            watch = System.Diagnostics.Stopwatch.StartNew();
            Graph g = new Graph(nodes, edges);


            foreach (Node n in nodes)
            {
                n.SharedWithHumans(HumanPathogens, AllPathogens);
                if (n.PathogensSharedWithHumans > 0)
                    if (!sharing.Contains(n.EID2ID))
                        sharing.Add(n.EID2ID);

                n.PrepareSharedPathogens(g);
                alli += n.Pathogens.Count;
                alln += n.SharedPathogens.Count;

                allz += n.SharedPathogens.Intersect(HumanPathogens).ToList().Count;
                foreach (long p in n.Pathogens)
                {
                    if (!allp.Contains(p))
                        allp.Add(p);
                    if (HumanPathogens.Contains(p))
                        if (!zoonosesP.Contains(p))
                            zoonosesP.Add(p);
                }
            }


            string s = String.Format("{0}\t{1}\t{2}\t{3}\t{4}\t{5}\t{6}\t{7}\t{8}",
                P, PT, nodes.Count, sharing.Count, allp.Count, zoonosesP.Count, alli,
               alln, allz);

            return s;
        }

        static public string GenerateNetworkstats_string(IList<Node> nodes, string P, string PT)
        {
            IList<long> AllPathogens = Helper.AllPathogens(nodes);
            IList<long> HumanPathogens = TraitsProcessor.GetHumanPathogen(P);
            IList<long> sharing = new List<long>();

            IList<long> allp = new List<long>();
            long alli = 0;
            long alln = 0;
            long allz = 0;
            IList<long> zoonosesP = new List<long>();


            Console.WriteLine("generating edges");
            IList<Edge> edges = NetworkBuilder.GenerateEdges(nodes);

            Console.WriteLine("generating graph");
            Graph g = new Graph(nodes, edges);


            foreach (Node n in nodes)
            {
                n.SharedWithHumans(HumanPathogens, AllPathogens);
                if (n.PathogensSharedWithHumans > 0)
                    if (!sharing.Contains(n.EID2ID))
                        sharing.Add(n.EID2ID);

                n.PrepareSharedPathogens(g);
                alli += n.Pathogens.Count;
                alln += n.SharedPathogens.Count;

                allz += n.SharedPathogens.Intersect(HumanPathogens).ToList().Count;
                foreach (long p in n.Pathogens)
                {
                    if (!allp.Contains(p))
                        allp.Add(p);
                    if (HumanPathogens.Contains(p))
                        if (!zoonosesP.Contains(p))
                            zoonosesP.Add(p);
                }
            }


            string s = String.Format("{0}\t{1}\t{2}\t{3}\t{4}\t{5}\t{6}\t{7}\t{8}",
                P, PT, nodes.Count, sharing.Count, allp.Count, zoonosesP.Count, alli,
               alln, allz);

            return s;
        }
    }
}
