﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace NetworkBuilder
{
    public class Randomiser
    {
        private static int total = 1000;
        public static long[][] GetInteractions(string P, string PT)
        {
            Console.WriteLine("generating nodes");
            var watch = System.Diagnostics.Stopwatch.StartNew();
            IList<Node> nodes = NetworkBuilder.GenerateNodes(P, PT);
            watch.Stop();
            string elapsed = watch.Elapsed.ToString();
            Console.WriteLine(elapsed);


            IList<long[]> interactions = new List<long[]>();
            foreach (Node node in nodes)
            {
                foreach (long p in node.Pathogens)
                {
                    interactions.Add(new long[2]);
                    int last = interactions.Count - 1;
                    interactions[last][0] = node.EID2ID;
                    interactions[last][1] = p;
                }
            }
            return interactions.ToArray();
        }

        static public IList<Node> GenerateEmptyNodes(string P, string PT)
        {
            IDictionary<long, string> Order = TraitsProcessor.OrderFamily(3);

            IList<long> Pathogens = PathogensExtractor.ExtractSubType(P, PT);


            string input = Globals.VertebratesDataSet;

            IList<Node> nodes = new List<Node>();
            string[] Data = File.ReadAllLines(input);

            for (int i = 1; i < Data.Length; i++)
            {
                try
                {
                    nodes = Node.Update(nodes, "mammals", false, Data[i], Pathogens);
                }
                catch (Exception e)
                {
                    Console.WriteLine(e);
                }
            }

            //node attributes
            foreach (Node node in nodes)
            {
                if (Order.ContainsKey(node.EID2ID))
                {
                    node.Order = Order[node.EID2ID];
                }
                node.Pathogens = new List<long>();
            }
            return nodes;
        }

        public static void Run(string pathogen, string subtype)
        {
          
            for (int i = 1; i <= total; i++)
            {
                //string nodes_file = String.Format(@"{0}/nodes.csv", IOHelper.GenerateEnumeratorFolder(pathogen, subtype, i));
                string edges_file = String.Format(@"{0}/edges.csv", IOHelper.GenerateEnumeratorFolder(pathogen, subtype, i));

                if (!File.Exists(edges_file))
                {
                    var watch = System.Diagnostics.Stopwatch.StartNew();
                    IList<Node> nodes = GenerateEmptyNodes(pathogen, subtype);
                    watch.Stop();
                    string elapsed = watch.Elapsed.ToString();
                    Console.WriteLine(elapsed);

                    long[][] interactions = GetInteractions(pathogen, subtype);
                    interactions = Enumerator.Enumerate(interactions);
                    nodes = Node.UpdateInteractions(nodes, interactions);

                    IList<Edge> edges = NetworkBuilder.GenerateEdges(nodes);
                    Console.WriteLine("outputting edges");

                    TextWriter tw_edges = new StreamWriter(edges_file, false);
                    tw_edges.WriteLine(Globals.EdgesHeader);
                    tw_edges.Flush();


                    foreach (Edge e in edges)
                    {
                        e.Write(tw_edges, nodes);
                    }
                 

                    /*Console.WriteLine("generating graph");
                    watch = System.Diagnostics.Stopwatch.StartNew();
                    Graph g = new Graph(nodes, edges);
                    Console.WriteLine(elapsed);


                    Console.WriteLine("calculating diversity &centralities");
                    watch = System.Diagnostics.Stopwatch.StartNew();
                    IList<long> AllPathogens = Helper.AllPathogens(nodes);
                    IList<long> HumanPathogens = TraitsProcessor.GetHumanPathogen(pathogen);

                    foreach (Node n in nodes)
                    {
                        n.PrepareSharedPathogens(g);
                        n.SharedWithHumans(HumanPathogens, AllPathogens);
                        n.Centrality(g);

                    }
                    elapsed = watch.Elapsed.ToString();
                    Console.WriteLine(elapsed);

                    Console.WriteLine("outputting nodes");
                    TextWriter tw_nodes = new StreamWriter(nodes_file, false);
                    tw_nodes.WriteLine(Globals.NodesHeader);
                    tw_nodes.Flush();

                    foreach (Node n in nodes)
                    {
                        n.WriteFullNode(tw_nodes);
                    }
                    */
                }
            }
        }

        public static void Run_Basic(string pathogen, string subtype)
        {

            string folder = @"../../output/permutations/basics";
            string output_file = String.Format(@"{0}/{1}.txt", folder, subtype);
            TextWriter tw = new StreamWriter(output_file, false);
            tw.WriteLine(String.Format("{0}\t{1}\t{2}\t{3}\t{4}\t{5}\t{6}\t{7}\t{8}",
               "pathogen", "subtype", "nodes", "shared_human_nodes", "pathogens", "zoonoses", "interactions",
               "interactions_shared", "interactions_zoonoses"));
            tw.Flush();
            string s = PaperHelper.GenerateNetworkstats_string(pathogen, subtype);
            tw.WriteLine(s);
            tw.Flush();
            for (int i = 1; i <= total; i++)
            {
                
                      
                    var watch = System.Diagnostics.Stopwatch.StartNew();
                    IList<Node> nodes = GenerateEmptyNodes(pathogen, subtype);
                    watch.Stop();
                    string elapsed = watch.Elapsed.ToString();
                    Console.WriteLine(elapsed);

                    long[][] interactions = GetInteractions(pathogen, subtype);
                    interactions = Enumerator.Enumerate(interactions);
                    nodes = Node.UpdateInteractions(nodes, interactions);

                string ss = PaperHelper.GenerateNetworkstats_string(nodes, pathogen, subtype);
                tw.WriteLine(ss);
                tw.Flush();
            }
        }
    }
}
